(* 
 * Copyright (c) 2001, 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: gontutil.mli,v 1.15 2002/02/18 11:26:51 malekith Exp $
 *
 * Module interface: Gontutil
 * Description: Gont program tree representation + various utilities.
 *)

open Gonttree


(*** Exceptions ***)

(* Oh well... this is also used for type errors and so on... possibly
 * name should be changed... *)
exception Syntax_error of location * string

(* Some general error, raised when we don't know the actual location. *)
exception Error of string

(* Go figure... *)
exception Eof


(*** Utils. ***)

(* Reset compiler state. This generally just resets sequence counters. *)
val reset : unit -> unit

(* Return unique name prefixed with string. *)
val seq_name : id -> id

(* Output debugging (trace) message. *)
val debug : string -> unit

(* An alias for Printf.sprintf. *)
val xf : ('a, unit, string) format -> 'a

(* Display warning. *)
val warn : location * string -> unit

(* Number of warning displayed so far. *)
val warnings_count : int ref

(* Display error message. *)
val err_msg : location * string -> unit

(* Raise Syntax_error. *)
val err : location * string -> 'a

(* 'a is type for context, 'b is input value, and 'c is output value. *)
val list_apply : ('a * 'b -> 'a * 'c) -> 'a * 'b list -> 'a * 'c list

(* Apply given function to type recursively. Function should not call
 * type_walk. *)
val type_walk : (ty -> ty) -> ty -> ty

(* Return list of free type variables in type. It does not go into
 * named types. *)
val simple_fv : ty -> id list

(* Make function type with tyvars set with simple_fv. Utility used by
 * Parser and Gontbind modules. *)
val make_fn_type : (ty * (parm_record list)) -> ty

(* Check whatever symbol has given property. *)
val has_property : symbol -> symbol_property -> bool

(* Assign given property to symbol (if it didn't already have it). *)
val set_property : symbol -> symbol_property -> unit

(* Return typedef of union (works only for symbols with property 
 * IsUnionField). *)
val union_typedef : symbol -> typedef

(* Return sequential number assigned to given union field. *)
val union_field_id : symbol -> int

(* Return module of given symbol. Raise Invalid_argument if symbol name
 * contains no module. *)
val sym_module : symbol -> id
;;
