/* 
 * Copyright (c) 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: slist.g,v 1.1 2002/02/18 11:28:23 malekith Exp $
 *
 * Standard library for Gont, Slist module implementation.
 */

open Slist

int length(<'a>t l)
{
	int n = 0;
	
	while (l != null) {
		n = n + 1;
		l = l.next;
	}

	return n;
}

void iter(*('a) -> void f, <'a>t l)
{
	while (l != null) {
		f(l.data);
		l = l.next;
	}
}

<'b>t map(*('a) -> 'b f, <'a>t l)
{
	if (l == null)
		return null;
	else
		return ({data = f(l.data), next = map(f, l.next)});
}

<'a>t concat(<'a>t x, <'a>t y)
{
	if (x == null)
		return y;
	else
		return ({data = x.data, next = concat(x.next, y)});
}

<'a>t filter(*('a) -> bool f, <'a>t l)
{
	if (l == null) {
		return null;
	} else {
		if (f(l.data))
			return ({data = l.data, next = filter(f, l.next)});
		else
			return filter(f, l.next);
	}
}

'a fold_left (*('a, 'b) -> 'a f, 'a init, <'b>t l) 
{
	if (l == null)
		return init;
	else
		return fold_left(f, f(init, l.data), l.next);
}

<'a>t qsort(*('a, 'a) -> int cmp, <'a>t l)
{
	'a m;

	if (l == null) {
		return l;
	} else {
		m = l.data;
		
		<'a>t le = filter(fun ('a x) -> bool is 
						(cmp(x, m) <= 0), l.next);
		<'a>t gt = filter(fun ('a x) -> bool is 
						(cmp(x, m) > 0), l.next);
		return concat(qsort(cmp, le), 
 			      ({data = m, next = qsort(cmp, gt)}));
	}
}
